@@ -19,7 +19,7 @@
 #ifndef PHP_EVENT_H
 #define PHP_EVENT_H
 
-#define PHP_EVENT_VERSION "2.3.0"
+#define PHP_EVENT_VERSION "2.4.0"
 
 #define PHP_EVENT_SUN_PREFIX "unix:"
 
@@ -28,6 +28,13 @@ extern zend_module_entry event_module_entry;
 
 #include "src/common.h"
 
+#if PHP_MAJOR_VERSION > 5 || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1)
+# define can_handle_soft_dependency_on_SPL 1
+#endif
+
+zend_class_entry *php_event_get_exception(void);
+zend_class_entry *php_event_get_exception_base(int root TSRMLS_DC);
+
 PHP_MINIT_FUNCTION(event);
 PHP_MSHUTDOWN_FUNCTION(event);
 PHP_RINIT_FUNCTION(event);
@@ -57,6 +64,12 @@ ZEND_EXTERN_MODULE_GLOBALS(event)
 # define PHP_EVENT_ASSERT(x)
 #endif
 
+#ifdef PHP_EVENT_NS
+# define PHP_EVENT_NS_NAME(name) PHP_EVENT_NS "\\" #name
+#else
+# define PHP_EVENT_NS_NAME(name) #name
+#endif
+
 #if PHP_VERSION_ID >= 50300
 # define PHP_EVENT_FCI_ADDREF(pfci)       \
 {                                         \
