@@ -19,7 +19,7 @@
 #ifndef PHP_EVENT_H
 #define PHP_EVENT_H
 
-#define PHP_EVENT_VERSION "2.3.0"
+#define PHP_EVENT_VERSION "2.4.0"
 
 #define PHP_EVENT_SUN_PREFIX "unix:"
 
@@ -28,6 +28,10 @@ extern zend_module_entry event_module_entry;
 
 #include "src/common.h"
 
+zend_class_entry *php_event_get_exception_base(int root);
+zend_class_entry *php_event_get_exception(void);
+
+
 PHP_MINIT_FUNCTION(event);
 PHP_MSHUTDOWN_FUNCTION(event);
 PHP_RINIT_FUNCTION(event);
@@ -44,6 +48,12 @@ ZEND_TSRMLS_CACHE_EXTERN();
 # define EVENT_G(v) (event_globals.v)
 #endif
 
+#ifdef PHP_EVENT_NS
+# define PHP_EVENT_NS_NAME(name) PHP_EVENT_NS "\\" #name
+#else
+# define PHP_EVENT_NS_NAME(name) #name
+#endif
+
 #ifdef PHP_EVENT_DEBUG
 # define PHP_EVENT_ASSERT(x) assert(x)
 #else
