@@ -19,6 +19,14 @@
 #include "fe.h"
 #include "priv.h"
 
+#ifdef ZEND_TYPE_ENCODE_CLASS_CONST
+# define PHP_EVENT_ARG_OBJ_INFO(pass_by_ref, name, classname, allow_null) \
+	{ #name, ZEND_TYPE_ENCODE_CLASS_CONST(PHP_EVENT_NS_NAME(classname), allow_null), pass_by_ref, 0 },
+#else
+# define PHP_EVENT_ARG_OBJ_INFO(pass_by_ref, name, classname, allow_null) \
+	{ #name, PHP_EVENT_NS_NAME(classname), IS_OBJECT, pass_by_ref, allow_null, 0 },
+#endif
+
 /* {{{ ARGINFO */
 ZEND_BEGIN_ARG_INFO(arginfo_event__void, 0)
 ZEND_END_ARG_INFO();
@@ -44,7 +52,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_event_base_loopexit, 0, 0, 0)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_event__construct, 0, 0, 4)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, fd)
 	ZEND_ARG_INFO(0, what)
 	ZEND_ARG_INFO(0, cb)
@@ -52,7 +60,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_event__construct, 0, 0, 4)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_event_set, 0, 0, 2)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, fd)
 	ZEND_ARG_INFO(0, what)
 	ZEND_ARG_INFO(0, cb)
@@ -80,20 +88,20 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_event_pending, 0, 0, 1)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_evtimer_new, 0, 0, 2)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, cb)
 	ZEND_ARG_INFO(0, arg)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_evtimer_set, 0, 0, 2)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, cb)
 	ZEND_ARG_INFO(0, arg)
 ZEND_END_ARG_INFO();
 
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_evsignal_new, 0, 0, 3)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, signum)
 	ZEND_ARG_INFO(0, cb)
 	ZEND_ARG_INFO(0, arg)
@@ -114,7 +122,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent__events, 0, 0, 1)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent__construct, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, socket)
 	ZEND_ARG_INFO(0, options)
 	ZEND_ARG_INFO(0, readcb)
@@ -159,7 +167,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent_read, 0, 0, 1)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent_pair_new, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, options)
 ZEND_END_ARG_INFO();
 
@@ -175,23 +183,23 @@ ZEND_END_ARG_INFO();
 #ifdef HAVE_EVENT_OPENSSL_LIB
 ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent_ssl_filter, 0, 0, 4)
 	ZEND_ARG_INFO(0, unused)
-	ZEND_ARG_OBJ_INFO(0, underlying, EventBufferEvent, 0)
-	ZEND_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, underlying, EventBufferEvent, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
 	ZEND_ARG_INFO(0, state)
 	ZEND_ARG_INFO(0, options)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent_create_ssl_filter, 0, 0, 3)
-	ZEND_ARG_OBJ_INFO(0, underlying, EventBufferEvent, 0)
-	ZEND_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, underlying, EventBufferEvent, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
 	ZEND_ARG_INFO(0, state)
 	ZEND_ARG_INFO(0, options)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_bufferevent_ssl_socket, 0, 0, 4)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, socket)
-	ZEND_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
 	ZEND_ARG_INFO(0, state)
 	ZEND_ARG_INFO(0, options)
 ZEND_END_ARG_INFO();
@@ -264,11 +272,11 @@ ZEND_END_ARG_INFO();
 
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_event_evhttp_connection__construct, 0, 0, 5)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, dns_base)
 	ZEND_ARG_INFO(0, address)
 	ZEND_ARG_INFO(0, port)
-	ZEND_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_event_evhttp_connection_get_peer, 0, 0, 2)
@@ -297,8 +305,8 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_event_evhttp_connection_set_retries, 0, 0, 1)
 ZEND_END_ARG_INFO();
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_event_http__construct, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
-	ZEND_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, ctx, EventSslContext, 0)
 ZEND_END_ARG_INFO();
 
 
@@ -422,7 +430,7 @@ ZEND_END_ARG_INFO();
 /* {{{ ARGINFO for extra API */
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_evdns__construct, 0, 0, 2)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, initialize)
 ZEND_END_ARG_INFO();
 
@@ -454,7 +462,7 @@ ZEND_END_ARG_INFO();
 
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_evconnlistener__construct, 0, 0, 6)
-	ZEND_ARG_OBJ_INFO(0, base, EventBase, 0)
+	PHP_EVENT_ARG_OBJ_INFO(0, base, EventBase, 0)
 	ZEND_ARG_INFO(0, cb)
 	ZEND_ARG_INFO(0, data)
 	ZEND_ARG_INFO(0, flags)
@@ -509,6 +517,8 @@ const zend_function_entry php_event_ce_functions[] = {/* {{{ */
 
 const zend_function_entry php_event_base_ce_functions[] = {/* {{{ */
 	PHP_ME(EventBase, __construct,        arginfo_event_base__construct,    ZEND_ACC_PUBLIC  | ZEND_ACC_CTOR)
+	PHP_ME(EventBase, __sleep,            arginfo_event_base_void,          ZEND_ACC_PUBLIC  | ZEND_ACC_FINAL)
+	PHP_ME(EventBase, __wakeup,           arginfo_event_base_void,          ZEND_ACC_PUBLIC  | ZEND_ACC_FINAL)
 	PHP_ME(EventBase, getMethod,          arginfo_event_base_void,          ZEND_ACC_PUBLIC)
 	PHP_ME(EventBase, getFeatures,        arginfo_event__void,              ZEND_ACC_PUBLIC)
 	PHP_ME(EventBase, priorityInit,       arginfo_event_base_priority_init, ZEND_ACC_PUBLIC)
@@ -533,7 +543,9 @@ const zend_function_entry php_event_base_ce_functions[] = {/* {{{ */
 /* }}} */
 
 const zend_function_entry php_event_config_ce_functions[] = {/* {{{ */
-	PHP_ME(EventConfig, __construct,     arginfo_event__void,                   ZEND_ACC_PUBLIC  | ZEND_ACC_CTOR)
+	PHP_ME(EventConfig, __construct,     arginfo_event__void,                   ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
+	PHP_ME(EventConfig, __sleep,         arginfo_event__void,                   ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(EventConfig, __wakeup,        arginfo_event__void,                   ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(EventConfig, avoidMethod,     arginfo_event_config_avoid_method,     ZEND_ACC_PUBLIC)
 	PHP_ME(EventConfig, requireFeatures, arginfo_event_config_require_features, ZEND_ACC_PUBLIC)
 #if LIBEVENT_VERSION_NUMBER >= 0x02010000
@@ -610,7 +622,7 @@ const zend_function_entry php_event_buffer_ce_functions[] = {/* {{{ */
 /* }}} */
 
 const zend_function_entry php_event_util_ce_functions[] = {/* {{{ */
-	PHP_ABSTRACT_ME(EventUtil, __construct, NULL)
+	PHP_ME(EventUtil, __construct, arginfo_event__void, ZEND_ACC_PRIVATE)
 
 	PHP_ME(EventUtil, getLastSocketErrno, arginfo_event_socket_1, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
 	PHP_ME(EventUtil, getLastSocketError, arginfo_event_socket_1, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
@@ -647,7 +659,9 @@ const zend_function_entry php_event_dns_base_ce_functions[] = {
 };
 
 const zend_function_entry php_event_http_conn_ce_functions[] = {
-	PHP_ME(EventHttpConnection, __construct,       arginfo_event_evhttp_connection__construct,        ZEND_ACC_PUBLIC  | ZEND_ACC_CTOR)
+	PHP_ME(EventHttpConnection, __construct,       arginfo_event_evhttp_connection__construct,        ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
+	PHP_ME(EventHttpConnection, __sleep,           arginfo_event__void,                               ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(EventHttpConnection, __wakeup,          arginfo_event__void,                               ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(EventHttpConnection, getBase,           arginfo_event__void,                               ZEND_ACC_PUBLIC)
 	PHP_ME(EventHttpConnection, getPeer,           arginfo_event_evhttp_connection_get_peer,          ZEND_ACC_PUBLIC)
 	PHP_ME(EventHttpConnection, setLocalAddress,   arginfo_event_evhttp_connection_set_local_address, ZEND_ACC_PUBLIC)
@@ -663,7 +677,9 @@ const zend_function_entry php_event_http_conn_ce_functions[] = {
 };
 
 const zend_function_entry php_event_http_ce_functions[] = {
-	PHP_ME(EventHttp, __construct,        arginfo_event_http__construct,          ZEND_ACC_PUBLIC  | ZEND_ACC_CTOR)
+	PHP_ME(EventHttp, __construct,        arginfo_event_http__construct,          ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
+	PHP_ME(EventHttp, __sleep,            arginfo_event__void,                    ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(EventHttp, __wakeup,           arginfo_event__void,                    ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(EventHttp, accept,             arginfo_event_http_accept,              ZEND_ACC_PUBLIC)
 	PHP_ME(EventHttp, bind,               arginfo_event_http_bind,                ZEND_ACC_PUBLIC)
 	PHP_ME(EventHttp, setCallback,        arginfo_event_http_set_callback,        ZEND_ACC_PUBLIC)
@@ -681,6 +697,8 @@ const zend_function_entry php_event_http_ce_functions[] = {
 const zend_function_entry php_event_http_req_ce_functions[] = {
 	PHP_ME(EventHttpRequest, __construct, arginfo_event_http_req__construct, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
 
+	PHP_ME(EventHttpRequest, __sleep,          arginfo_event__void,                     ZEND_ACC_PUBLIC  | ZEND_ACC_FINAL)
+	PHP_ME(EventHttpRequest, __wakeup,         arginfo_event_base_void,                 ZEND_ACC_PUBLIC  | ZEND_ACC_FINAL)
 	PHP_ME(EventHttpRequest, free,             arginfo_event__void,                     ZEND_ACC_PUBLIC)
 	PHP_ME(EventHttpRequest, getCommand,       arginfo_event__void,                     ZEND_ACC_PUBLIC)
 	PHP_ME(EventHttpRequest, getHost,          arginfo_event__void,                     ZEND_ACC_PUBLIC)
@@ -710,7 +728,9 @@ const zend_function_entry php_event_http_req_ce_functions[] = {
 };
 
 const zend_function_entry php_event_listener_ce_functions[] = {
-	PHP_ME(EventListener, __construct,      arginfo_evconnlistener__construct,   ZEND_ACC_PUBLIC  | ZEND_ACC_CTOR)
+	PHP_ME(EventListener, __construct,      arginfo_evconnlistener__construct,   ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
+	PHP_ME(EventListener, __sleep,          arginfo_event__void,                 ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(EventListener, __wakeup,         arginfo_event__void,                 ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(EventListener, free,             arginfo_event__void,                 ZEND_ACC_PUBLIC)
 	PHP_ME(EventListener, enable,           arginfo_event__void,                 ZEND_ACC_PUBLIC)
 	PHP_ME(EventListener, disable,          arginfo_event__void,                 ZEND_ACC_PUBLIC)
